@@ -70,7 +70,7 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
             // need to make sure we have a specific AppId shared by all peers
             identity::OfflinePlayground::new(ditto_root, app_id)
         })?
-        .with_minimum_log_level(CLogLevel::Info)
+        .with_minimum_log_level(LogLevel::Info)
         .with_transport_config(|_identity| -> TransportConfig {
             let mut transport_config = TransportConfig::new();
             transport_config.listen.tcp.enabled = true;
@@ -128,7 +128,7 @@ fn main() -> Result<(), Box<dyn std::error::Error>> {
         }
     };
     // downgrade our logging output before running the query
-    Ditto::set_minimum_log_level(CLogLevel::Debug);
+    Ditto::set_minimum_log_level(LogLevel::Debug);
     // Find and report on all cars
     let _lq = collection.find_all().observe(event_handler);
     for car_cbor in rx.iter() {
