@@ -8,6 +8,7 @@ use std::{
 use ffi_sdk::{BoxedAuthClient, BoxedLoginProvider};
 
 use crate::{
+    ditto::{Ditto, DittoFields},
     error::{DittoError, ErrorKind},
     transport::Transports,
     utils::prelude::*,
@@ -104,12 +105,12 @@ impl LoginProviderCtx {
                 if seconds_remaining == 0 {
                     ctx_ref
                         .auth_event_handler
-                        .authentication_required(authn.clone());
+                        .authentication_required(authn.retain());
                 } else {
                     let duration = Duration::from_secs(seconds_remaining.into());
                     ctx_ref
                         .auth_event_handler
-                        .authentication_expiring_soon(authn.clone(), duration);
+                        .authentication_expiring_soon(authn.retain(), duration);
                 }
             }
             None => ctx_ref.cached_expiry_time = Some(seconds_remaining),
@@ -122,11 +123,11 @@ impl LoginProviderCtx {
                 // easier than trying to call across the FFI
                 if time == 0 {
                     self.auth_event_handler
-                        .authentication_required(authn.clone());
+                        .authentication_required(authn.retain());
                 } else {
                     let duration = Duration::from_secs(time.into());
                     self.auth_event_handler
-                        .authentication_expiring_soon(authn.clone(), duration);
+                        .authentication_expiring_soon(authn.retain(), duration);
                 }
                 self.cached_expiry_time = None; // set to None after calling
                                                 // either method
@@ -180,24 +181,30 @@ impl ValidityListener {
     }
 }
 
+impl RefCounted for DittoAuthenticator {}
+// Cloning this will just increment the count on the inner Arc<BoxedAuthClient>,
+// and allow the user to provide actual credentials at a later time
+#[derive(Clone)]
 /// Handle to trigger actual authentication requests
-#[derive(Clone)] // Cloning this will just increment the count on the inner Arc<BoxedAuthClient>,
-                 // and allow the user to provide actual credentials at a later time
 pub struct DittoAuthenticator {
     auth_client: Arc<BoxedAuthClient>,
+    pub(crate) ditto_fields: std::sync::Weak<DittoFields>,
 }
 
 impl DittoAuthenticator {
     pub fn new(auth_client: Arc<BoxedAuthClient>) -> Self {
-        DittoAuthenticator { auth_client }
+        DittoAuthenticator {
+            auth_client,
+            ditto_fields: std::sync::Weak::<DittoFields>::new(),
+        }
     }
 
     pub(crate) fn auth_client(&self) -> Arc<BoxedAuthClient> {
-        self.auth_client.clone()
+        self.auth_client.retain()
     }
 
     /// Asks the Ditto AuthClient to make an Auth Request to the configured
-    /// Identitie's auth url  with a single token parameter.
+    /// Identity's auth url  with a single token parameter.
     ///
     /// * `token` - An auth or API token you have configured
     /// * `provider` - The name of an authentication provider web hook you have
@@ -278,6 +285,24 @@ impl DittoAuthenticator {
         }
     }
 
+    /// Log out of Ditto
+    /// Shutdown all replication sessions and remove any cached authentication
+    /// credentials. This does *not* remove the local data store.
+    pub fn logout<R>(&self, cleanup: impl FnOnce(Ditto) -> R) -> Result<R, DittoError> {
+        let fields = self
+            .ditto_fields
+            .upgrade()
+            .expect("fields hasn't been dropped yet");
+        let ditto = Ditto::new_with_fields(fields);
+        let status = unsafe { ffi_sdk::ditto_auth_client_logout(&*self.auth_client) };
+        if status != 0 {
+            return Err(DittoError::from_ffi(ErrorKind::Authentication));
+        }
+        ditto.stop_sync();
+        let ret = cleanup(ditto);
+        Ok(ret)
+    }
+
     /// Query whether Ditto has a valid authentication token.
     ///
     /// This will only be `true` when using an `Online` or an
@@ -300,15 +325,4 @@ impl DittoAuthenticator {
             s
         })
     }
-
-    /// Log out of Ditto
-    /// Shutdown all replication sessions and remove any cached authentication
-    /// credentials. This does *not* remove the local data store.
-    pub fn logout(&self) -> Result<(), DittoError> {
-        let status = unsafe { ffi_sdk::ditto_auth_client_logout(&*self.auth_client) };
-        match status {
-            0 => Ok(()),
-            _ => Err(DittoError::from_ffi(ErrorKind::Authentication)),
-        }
-    }
 }
