@@ -20,6 +20,9 @@ use crate::{
     utils::prelude::*,
 };
 
+#[doc(inline)]
+pub use ffi_sdk::CLogLevel as LogLevel;
+
 /// The entry point for accessing Ditto-related functionality
 /// This struct is generally a handle and interface to ditto-functionality
 /// which operates in background threads.
@@ -179,11 +182,11 @@ impl Ditto {
         }
     }
 
-    pub fn get_minimum_log_level() -> ffi_sdk::CLogLevel {
+    pub fn get_minimum_log_level() -> LogLevel {
         unsafe { ffi_sdk::ditto_logger_minimum_log_level_get() }
     }
 
-    pub fn set_minimum_log_level(log_level: ffi_sdk::CLogLevel) {
+    pub fn set_minimum_log_level(log_level: LogLevel) {
         unsafe {
             ffi_sdk::ditto_logger_minimum_log_level(log_level);
         }
@@ -352,7 +355,7 @@ impl Ditto {
             ))
             .with_identity(|ditto_root| identity::OfflinePlayground::new(ditto_root, app_id))
             .expect("Invalid Ditto Identity")
-            .with_minimum_log_level(CLogLevel::Info)
+            .with_minimum_log_level(LogLevel::Info)
             .build()
             .expect("Failed to build Ditto Instance")
     }
