@@ -1,3 +1,4 @@
+#[allow(unused_imports)]
 #[macro_use]
 extern crate ext_trait;
 extern crate ffi_sdk;
@@ -21,3 +22,6 @@ pub mod store;
 pub mod subscription;
 
 pub mod transport;
+
+#[cfg(test)]
+pub(in crate) mod test_helpers;
