@@ -1,6 +1,7 @@
 pub use crate::{
     auth::DittoAuthenticationEventHandler,
-    ditto::{builder::DittoBuilder, AppId, Ditto, SiteId},
+    ditto::{builder::DittoBuilder, AppId, Ditto, LogLevel, SiteId},
+    error::DittoError,
     ffi_sdk::{BoxedDitto, BoxedDocument, CLogLevel},
     fs::{DittoRoot, PersistentRoot, TempRoot},
     identity::{
@@ -16,7 +17,10 @@ pub use crate::{
         collections::{
             event::CollectionsEvent, pending_collections_operation::CollectionsEventHandler,
         },
-        live_query::{EventHandler, LiveQuery, LiveQueryEvent, LiveQueryMove},
+        live_query::{
+            EventHandler, LiveQuery, LiveQueryEvent, LiveQueryMove, SingleDocumentEventHandler,
+            SingleDocumentLiveQueryEvent,
+        },
         timeseries::TimeSeries,
         Store, WriteStrategy,
     },
