@@ -19,9 +19,39 @@ pub struct ScopedCollection<'coll, 'batch> {
 }
 
 impl ScopedCollection<'_, '_> {
-    pub fn insert<V: ::serde::Serialize>(
+    /// Inserts a new document into the collection and returns its ID. If the
+    /// document already exists, the provided document content will be merged
+    /// with the existing document's content.
+    pub fn upsert<V: ::serde::Serialize, C: Borrow<V>>(
         &'_ mut self,
-        content: impl Borrow<V>,
+        content: C,
+    ) -> Result<&'_ DocumentId, DittoError> {
+        self.insert_cbor(
+            ::serde_cbor::to_vec(content.borrow()).unwrap().as_slice(),
+            None,
+            WriteStrategy::Merge,
+        )
+    }
+
+    /// Inserts a new document into the collection and returns its ID. If the
+    /// document already exists, the behavior is determined by the provided
+    /// `write_strategy`.
+    pub fn upsert_with_strategy<V: ::serde::Serialize, C: Borrow<V>>(
+        &'_ mut self,
+        content: C,
+        write_strategy: WriteStrategy,
+    ) -> Result<&'_ DocumentId, DittoError> {
+        self.insert_cbor(
+            ::serde_cbor::to_vec(content.borrow()).unwrap().as_slice(),
+            None,
+            write_strategy,
+        )
+    }
+
+    #[deprecated(note = "Use `upsert` instead")]
+    pub fn insert<V: ::serde::Serialize, C: Borrow<V>>(
+        &'_ mut self,
+        content: C,
         id: Option<&DocumentId>,
         is_default: bool,
     ) -> Result<&'_ DocumentId, DittoError> {
@@ -37,9 +67,10 @@ impl ScopedCollection<'_, '_> {
         )
     }
 
-    pub fn insert_with_strategy<V: ::serde::Serialize>(
+    #[deprecated(note = "Use `upsert_with_strategy` instead")]
+    pub fn insert_with_strategy<V: ::serde::Serialize, C: Borrow<V>>(
         &'_ mut self,
-        content: impl Borrow<V>,
+        content: C,
         id: Option<&DocumentId>,
         write_strategy: WriteStrategy,
     ) -> Result<&'_ DocumentId, DittoError> {
@@ -87,10 +118,10 @@ impl<'coll, 'batch> ScopedCollection<'coll, 'batch> {
         }
     }
 
-    pub fn find_with_args<'find, 'order_by, V: ::serde::Serialize>(
+    pub fn find_with_args<'find, 'order_by, V: ::serde::Serialize, C: Borrow<V>>(
         self: &'find mut ScopedCollection<'coll, 'batch>,
         query: &'_ str,
-        query_args: impl Borrow<V>,
+        query_args: C,
     ) -> BatchCursorOperation<'find, 'coll, 'batch, 'order_by> {
         let query = char_p::new(query);
         BatchCursorOperation {
