@@ -8,7 +8,6 @@ mod common;
 
 #[test]
 fn exec_with_query_args() {
-    env_logger::init();
     let ditto = common::get_ditto().unwrap();
     let store = ditto.store();
     let collection_name = "test";
@@ -16,32 +15,16 @@ fn exec_with_query_args() {
 
     // Insert some documents that will match, and some that won't
     collection
-        .insert(
-            json!({"make": "Honda", "color": "red"}),
-            Some(&DocumentId::new(&"honda".to_string()).unwrap()),
-            false,
-        )
+        .upsert(json!({"_id": "honda", "make": "Honda", "color": "red"}))
         .unwrap();
     collection
-        .insert(
-            json!({"make": "Ford", "color": "blue"}),
-            Some(&DocumentId::new(&"ford".to_string()).unwrap()),
-            false,
-        )
+        .upsert(json!({"_id": "ford", "make": "Ford", "color": "blue"}))
         .unwrap();
     collection
-        .insert(
-            json!({"make": "Aston Martin", "color": "silver"}),
-            Some(&DocumentId::new(&"aston_martin".to_string()).unwrap()),
-            false,
-        )
+        .upsert(json!({"_id": "aston_martin", "make": "Aston Martin", "color": "silver"}))
         .unwrap();
     collection
-        .insert(
-            json!({"make": "Ferrari", "color": "red"}),
-            Some(&DocumentId::new(&"ferrari".to_string()).unwrap()),
-            false,
-        )
+        .upsert(json!({"_id": "ferrari", "make": "Ferrari", "color": "red"}))
         .unwrap();
 
     let _ = store.with_batched_write(|mut tx: ScopedStore<'_>| {
@@ -67,20 +50,16 @@ fn evict_and_remove_with_args() {
     let collection_name = "test";
     let collection = store.collection(collection_name).unwrap();
     let honda_id = collection
-        .insert(json!({"make": "Honda", "color": "red"}), None, false)
+        .upsert(json!({"make": "Honda", "color": "red"}))
         .unwrap();
     collection
-        .insert(json!({"make": "Ford", "color": "blue"}), None, false)
+        .upsert(json!({"make": "Ford", "color": "blue"}))
         .unwrap();
     let ferrari_id = collection
-        .insert(json!({"make": "Ferrari", "color": "red"}), None, false)
+        .upsert(json!({"make": "Ferrari", "color": "red"}))
         .unwrap();
     let lamborghini_id = collection
-        .insert(
-            json!({"make": "Lamborghini", "color": "yellow"}),
-            None,
-            false,
-        )
+        .upsert(json!({"make": "Lamborghini", "color": "yellow"}))
         .unwrap();
 
     let docs = collection
