@@ -7,7 +7,7 @@ use common::*;
 
 #[test]
 fn empty_name_is_err() {
-    let dk = common::get_ditto().unwrap();
+    let dk = get_ditto().unwrap();
     let store = dk.store();
     let coll = store.collection("");
     // Without an explicit call to drop,
@@ -18,35 +18,57 @@ fn empty_name_is_err() {
 
 #[test]
 fn whitespace_name_is_err() {
-    let ditto = common::get_ditto().unwrap();
+    let ditto = get_ditto().unwrap();
     let store = ditto.store();
     let collection = store.collection("   ");
     assert!(collection.is_err());
 }
 
 #[test]
-fn simple_insert() {
-    let ditto = common::get_ditto().unwrap();
+fn simple_upsert() {
+    let ditto = get_ditto().unwrap();
     let store = ditto.store();
-    let content = json!({"hello": "again"});
     let collection = store.collection("test").unwrap();
-    let id = collection.insert(content, None, false).unwrap();
+    let id = collection.upsert(json!({"hello": "again"})).unwrap();
     let doc = collection.find_by_id(id.to_owned()).exec().unwrap();
     assert_eq!(id, doc.id());
     assert_eq!(doc.get::<String>("hello").unwrap(), "again");
 }
 
 #[test]
-fn insert_overwrites_existing_content_by_default() {
-    let ditto = common::get_ditto().unwrap();
+fn upsert_merges_with_existing_content_by_default() {
+    let ditto = get_ditto().unwrap();
     let store = ditto.store();
     let collection = store.collection("test").unwrap();
     let custom_id = DocumentId::new(&"custom_id".to_string()).unwrap();
     let id1 = collection
-        .insert(json!({"some": "value"}), Some(&custom_id), false)
+        .upsert(json!({"some": "value", "_id": custom_id}))
         .unwrap();
     let id2 = collection
-        .insert(json!({"another": "thing"}), Some(&custom_id), false)
+        .upsert(json!({"another": "thing", "_id": custom_id}))
+        .unwrap();
+    let doc = collection.find_by_id(custom_id).exec().unwrap();
+    assert_eq!(id1, doc.id());
+    assert_eq!(id2, doc.id());
+    assert_eq!(doc.get::<String>("another").unwrap(), "thing");
+    assert_eq!(doc.get::<String>("another").unwrap(), "thing");
+    assert_eq!(doc.get::<String>("some").unwrap(), "value");
+}
+
+#[test]
+fn upsert_overwrites_existing_content_if_specified_in_write_strategy() {
+    let ditto = get_ditto().unwrap();
+    let store = ditto.store();
+    let collection = store.collection("test").unwrap();
+    let custom_id = DocumentId::new(&"custom_id".to_string()).unwrap();
+    let id1 = collection
+        .upsert(json!({"some": "value", "_id": custom_id}))
+        .unwrap();
+    let id2 = collection
+        .upsert_with_strategy(
+            json!({"another": "thing", "_id": custom_id}),
+            WriteStrategy::Overwrite,
+        )
         .unwrap();
     let doc = collection.find_by_id(custom_id).exec().unwrap();
     assert_eq!(id1, doc.id());
@@ -58,50 +80,23 @@ fn insert_overwrites_existing_content_by_default() {
 }
 
 #[test]
-fn insert_merges_with_existing_content_if_specified_in_options() {
-    let ditto = common::get_ditto().unwrap();
+fn upserting_default_data_if_a_document_with_a_matching_id_already_exists_is_a_no_op() {
+    let ditto = get_ditto().unwrap();
     let store = ditto.store();
     let collection = store.collection("test").unwrap();
     let custom_id = DocumentId::new(&"custom_id".to_string()).unwrap();
-    //@ditto/snippet-start insert
+
     let id1 = collection
-        .insert(json!({"some": "value"}), Some(&custom_id), false)
-        .unwrap();
-    //@ditto/snippet-end
-    let id2 = collection
-        .insert_with_strategy(
-            json!({"another": "thing"}),
-            Some(&custom_id),
-            WriteStrategy::Merge,
+        .upsert_with_strategy(
+            json!({"some": "value", "_id": custom_id}),
+            WriteStrategy::InsertDefaultIfAbsent,
         )
         .unwrap();
-    let doc = collection.find_by_id(custom_id).exec().unwrap();
-    assert_eq!(id1, doc.id());
-    assert_eq!(id2, doc.id());
-    assert_eq!(doc.get::<String>("another").unwrap(), "thing");
-    assert_eq!(doc.get::<String>("some").unwrap(), "value");
-}
 
-#[test]
-fn inserting_default_data_if_a_document_with_a_matching_id_already_exists_is_a_no_op() {
-    let ditto = common::get_ditto().unwrap();
-    let store = ditto.store();
-    let collection = store.collection("test").unwrap();
-    let custom_id = DocumentId::new(&"custom_id".to_string()).unwrap();
-
-    let id1 = collection
-        .insert(json!({"some": "value"}), Some(&custom_id), true)
-        .unwrap();
-
-    // Immediately try and insert some new default data using both supported
-    // methods
+    // Immediately try and insert some new default data
     let id2 = collection
-        .insert(json!({"different": "default data"}), Some(&custom_id), true)
-        .unwrap();
-    let id2b = collection
-        .insert_with_strategy(
-            json!({"other different": "default data"}),
-            Some(&custom_id),
+        .upsert_with_strategy(
+            json!({"different": "default data", "_id": custom_id}),
             WriteStrategy::InsertDefaultIfAbsent,
         )
         .unwrap();
@@ -112,19 +107,15 @@ fn inserting_default_data_if_a_document_with_a_matching_id_already_exists_is_a_n
     assert_eq!(doc.get::<String>("some").unwrap(), "value");
     assert!(doc.get::<String>("different").is_err());
 
-    // Perform an insert of a document containing *non* default data
+    // Perform an upsert of a document containing *non* default data
     let id3 = collection
-        .insert_with_strategy(json!({"a": "b"}), Some(&custom_id), WriteStrategy::Merge)
+        .upsert_with_strategy(json!({"a": "b", "_id": custom_id}), WriteStrategy::Merge)
         .unwrap();
 
-    // Again try and insert some new default data using both supported methods
+    // Again try and insert some new default data
     let id4 = collection
-        .insert(json!({"different": "default data"}), Some(&custom_id), true)
-        .unwrap();
-    let id4b = collection
-        .insert_with_strategy(
-            json!({"other different": "default data"}),
-            Some(&custom_id),
+        .upsert_with_strategy(
+            json!({"different": "default data", "_id": custom_id}),
             WriteStrategy::InsertDefaultIfAbsent,
         )
         .unwrap();
@@ -135,25 +126,24 @@ fn inserting_default_data_if_a_document_with_a_matching_id_already_exists_is_a_n
     assert!(doc.get::<String>("different").is_err());
     assert_eq!(id1, doc.id());
     assert_eq!(id2, doc.id());
-    assert_eq!(id2b, doc.id());
+    assert_eq!(id2, doc.id());
     assert_eq!(id3, doc.id());
     assert_eq!(id4, doc.id());
-    assert_eq!(id4b, doc.id());
+    assert_eq!(id4, doc.id());
 }
 
 #[test]
 fn inserts_a_document_if_no_preexisting_when_using_appropriate_strategy() {
-    let ditto = common::get_ditto().unwrap();
+    let ditto = get_ditto().unwrap();
     let store = ditto.store();
     let collection = store.collection("test").unwrap();
     let custom_id = DocumentId::new(&"custom_id".to_string()).unwrap();
     let id1 = collection
-        .insert(json!({"some": "value"}), Some(&custom_id), false)
+        .upsert(json!({"some": "value", "_id": custom_id}))
         .unwrap();
     let id2 = collection
-        .insert_with_strategy(
-            json!({"another": "thing"}),
-            Some(&custom_id),
+        .upsert_with_strategy(
+            json!({"another": "thing", "_id": custom_id}),
             WriteStrategy::InsertIfAbsent,
         )
         .unwrap();
@@ -166,17 +156,17 @@ fn inserts_a_document_if_no_preexisting_when_using_appropriate_strategy() {
 
 #[test]
 fn find_with_args() {
-    let ditto = common::get_ditto().unwrap();
+    let ditto = get_ditto().unwrap();
     let store = ditto.store();
     let collection = store.collection("test").unwrap();
     collection
-        .insert(json!({"make": "Honda", "color": "red"}), None, false)
+        .upsert(json!({"make": "Honda", "color": "red"}))
         .unwrap();
     collection
-        .insert(json!({"make": "Ford", "color": "blue"}), None, false)
+        .upsert(json!({"make": "Ford", "color": "blue"}))
         .unwrap();
     collection
-        .insert(json!({"make": "Ferrari", "color": "red"}), None, false)
+        .upsert(json!({"make": "Ferrari", "color": "red"}))
         .unwrap();
     let sort_param = ffi_sdk::COrderByParam {
         query_c_str: c!("make"),
@@ -193,69 +183,39 @@ fn find_with_args() {
 }
 
 #[test]
-fn insert_with_provided_id() {
-    let ditto = common::get_ditto().unwrap();
+fn upsert_with_provided_id() {
+    let ditto = get_ditto().unwrap();
     let store = ditto.store();
-    let content = json!({"hello": "again"});
     let collection = store.collection("test").unwrap();
     let custom_id = DocumentId::new(&"a".to_string()).unwrap();
-    let id = collection.insert(content, Some(&custom_id), false).unwrap();
+    let id = collection
+        .upsert(json!({"hello": "again", "_id": custom_id}))
+        .unwrap();
     let doc = collection.find_by_id(custom_id.to_owned()).exec().unwrap();
     assert_eq!(id, custom_id);
     assert_eq!(id, doc.id());
     assert_eq!(doc.get::<String>("hello").unwrap(), "again");
 }
 
-#[test]
-fn insert_with_id_in_document_content() {
-    let ditto = common::get_ditto().unwrap();
-    let store = ditto.store();
-    let collection = store.collection("test").unwrap();
-    let content = json!({"_id": DocumentId::new(&"abc".to_string()).unwrap(), "some": "value"});
-    let id = collection.insert(content, None, false).unwrap();
-    let expected_id = DocumentId::new(&"abc".to_string()).unwrap();
-    assert_eq!(id, expected_id);
-    let doc = collection
-        .find_by_id(expected_id.to_owned())
-        .exec()
-        .unwrap();
-    assert_eq!(expected_id, doc.id());
-    assert_eq!(doc.get::<String>("_id").unwrap(), expected_id.to_string());
-}
-
 #[test]
 fn get_id_subvalue_from_document_content() {
-    let ditto = common::get_ditto().unwrap();
+    let ditto = get_ditto().unwrap();
     let store = ditto.store();
     let collection = store.collection("test").unwrap();
     let custom_id =
-        DocumentId::new(&json!({ "nested": { "should": { "work": [123, 456] } } })).unwrap();
+        DocumentId::new(&json!({ "nested": { "should": { "work": ["hello", true] } } })).unwrap();
     let id = collection
-        .insert(json!({"a": "1"}), Some(&custom_id), false)
+        .upsert(json!({"a": "1", "_id": custom_id}))
         .unwrap();
     assert_eq!(id, custom_id);
     let doc = collection.find_by_id(custom_id.to_owned()).exec().unwrap();
     assert_eq!(custom_id, doc.id());
-    assert_eq!(doc.get::<u16>("_id.nested.should.work[1]").unwrap(), 456);
-}
-
-#[test]
-fn insert_with_provided_id_and_id_in_document_content() {
-    let ditto = common::get_ditto().unwrap();
-    let store = ditto.store();
-    let collection = store.collection("test").unwrap();
-    let custom_id = DocumentId::new(&"def".to_string()).unwrap();
-    let content = json!({"_id": DocumentId::new(&"abc".to_string()).unwrap(), "some": "value"});
-    let id = collection.insert(content, Some(&custom_id), false).unwrap();
-    assert_eq!(id, custom_id);
-    let doc = collection.find_by_id(custom_id.to_owned()).exec().unwrap();
-    assert_eq!(custom_id, doc.id());
-    assert_eq!(doc.get::<String>("_id").unwrap(), custom_id.to_string());
+    assert!(doc.get::<bool>("_id.nested.should.work[1]").unwrap());
 }
 
 #[test]
 fn roundtrip_with_document_ids_of_different_types() {
-    let ditto = common::get_ditto().unwrap();
+    let ditto = get_ditto().unwrap();
     let store = ditto.store();
     let collection = store.collection("test").unwrap();
 
@@ -278,7 +238,7 @@ fn roundtrip_with_document_ids_of_different_types() {
     raw_doc_ids.iter().for_each(|raw_id| {
         let doc_id = DocumentId::new(raw_id).unwrap();
         let returned_id = collection
-            .insert(json!({"hello": "again"}), Some(&doc_id), false)
+            .upsert(json!({"hello": "again", "_id": doc_id}))
             .unwrap();
         assert_eq!(doc_id, returned_id);
 
@@ -289,11 +249,10 @@ fn roundtrip_with_document_ids_of_different_types() {
 
 #[test]
 fn trying_to_update_document_id_via_id_key_fails() {
-    let ditto = common::get_ditto().unwrap();
+    let ditto = get_ditto().unwrap();
     let store = ditto.store();
-    let content = json!({"hello": "again"});
     let collection = store.collection("test").unwrap();
-    let id = collection.insert(content, None, false).unwrap();
+    let id = collection.upsert(json!({"hello": "again"})).unwrap();
 
     let update_res = collection.find_by_id(id.to_owned()).update(|mut_doc| {
         if let Some(doc) = mut_doc {
@@ -353,14 +312,13 @@ fn trying_to_update_document_id_via_id_key_fails() {
 /// of various types
 #[test]
 fn serialize_insert_roundtrip() {
-    let ditto = common::get_ditto().unwrap();
+    let ditto = get_ditto().unwrap();
     let store = ditto.store();
-    let content = TestType::random();
+    let id = DocumentId::new(&1).unwrap();
+    let content = TestType::random_with_id(Some(id.clone()));
     let collection = store.collection("test").unwrap();
-    let custom_id = DocumentId::new(&"a".to_string()).unwrap();
-    let id = collection
-        .insert(content.to_owned(), Some(&custom_id), false)
-        .unwrap();
+    let ret_id = collection.upsert(content.to_owned()).unwrap();
+    assert_eq!(ret_id, id);
     let doc = collection.find_by_id(id).exec().unwrap();
     let returned_content = doc.typed::<TestType>().unwrap();
     assert_eq!(content, returned_content);
@@ -368,23 +326,20 @@ fn serialize_insert_roundtrip() {
 
 #[test]
 fn inserting_invalid_doc_throws_error() {
-    let ditto = common::get_ditto().unwrap();
+    let ditto = get_ditto().unwrap();
     let store = ditto.store();
-    let content = json!(["hello", "again"]);
     let collection = store.collection("test").unwrap();
-    let result = collection.insert(content, None, false);
+    let result = collection.upsert(json!(["hello", "again"]));
     assert!(result.is_err());
 }
 
 #[test]
 fn simple_update() {
-    let ditto = common::get_ditto().unwrap();
+    let ditto = get_ditto().unwrap();
     let store = ditto.store();
     // this collection needs to be namespaced?
     let collection = store.collection("test").unwrap();
-    let content = TestType::random();
-
-    let id = collection.insert(content, None, false).unwrap();
+    let id = collection.upsert(TestType::random()).unwrap();
 
     collection
         .find_by_id(id.to_owned())
@@ -396,28 +351,26 @@ fn simple_update() {
         .unwrap();
 
     let updated = collection.find_by_id(id).exec().unwrap();
-
     let new_val = updated.get::<String>("something").unwrap();
-
     assert_eq!(new_val, "NewValue");
 }
 
 #[test]
 fn list_collection() {
-    let ditto = common::get_ditto().unwrap();
+    let ditto = get_ditto().unwrap();
     let store = ditto.store();
     let uuid = uuid::Uuid::new_v4().to_string();
     let collection = store.collection(&uuid).unwrap();
 
     // The insert order is no longer preserved
     let _id_a = collection
-        .insert(json!({"a": "valForA", "rank": 1}), None, false)
+        .upsert(json!({"a": "valForA", "rank": 1}))
         .unwrap();
     let _id_b = collection
-        .insert(json!({"b": "valForB", "rank": 2}), None, false)
+        .upsert(json!({"b": "valForB", "rank": 2}))
         .unwrap();
     let _id_c = collection
-        .insert(json!({"c": "valForC", "rank": 3}), None, false)
+        .upsert(json!({"c": "valForC", "rank": 3}))
         .unwrap();
 
     let sort_param = ffi_sdk::COrderByParam {
@@ -431,17 +384,17 @@ fn list_collection() {
 
 #[test]
 fn document_id() {
-    let ditto = common::get_ditto().unwrap();
+    let ditto = get_ditto().unwrap();
     let store = ditto.store();
     let collection = store.collection("test").unwrap();
 
     let cbor_doc_id = ::serde_cbor::value::to_value(::serde_json::json!({
         "a": {
             "_b_": [
-                123,
+                true,
                 "z",
                 {
-                    "c..": 42,
+                    "c..": ["?d!"],
                 },
             ],
         },
@@ -450,36 +403,31 @@ fn document_id() {
 
     let doc_id = DocumentId::new(&cbor_doc_id).unwrap();
     let returned_id = collection
-        .insert(json!({"hello": "again"}), Some(&doc_id), false)
+        .upsert(json!({"hello": "again", "_id": doc_id}))
         .unwrap();
     assert_eq!(doc_id, returned_id);
 
     let doc = collection.find_by_id(doc_id.clone()).exec().unwrap();
     assert_eq!(doc.id(), doc_id);
-
     assert_eq!(doc.id().value(), cbor_doc_id);
 }
 
 #[test]
 fn evict_remove_and_update_with_args() {
-    let ditto = common::get_ditto().unwrap();
+    let ditto = get_ditto().unwrap();
     let store = ditto.store();
     let collection = store.collection("test").unwrap();
     let honda_id = collection
-        .insert(json!({"make": "Honda", "color": "red"}), None, false)
+        .upsert(json!({"make": "Honda", "color": "red"}))
         .unwrap();
     let ford_id = collection
-        .insert(json!({"make": "Ford", "color": "blue"}), None, false)
+        .upsert(json!({"make": "Ford", "color": "blue"}))
         .unwrap();
     let ferrari_id = collection
-        .insert(json!({"make": "Ferrari", "color": "red"}), None, false)
+        .upsert(json!({"make": "Ferrari", "color": "red"}))
         .unwrap();
     let lamborghini_id = collection
-        .insert(
-            json!({"make": "Lamborghini", "color": "yellow"}),
-            None,
-            false,
-        )
+        .upsert(json!({"make": "Lamborghini", "color": "yellow"}))
         .unwrap();
 
     let docs = collection
